import time
import random

def linebreak(text, col=72):
    retval = ""
    count = 0
    for c in text:
        if count > col and c == " ":
            retval += "\n"
            count = 0
        else:
            retval +=c
            count += 1
    return retval

class State:
    def __init__(self, name, description):
        self.name = name
        self.description = description
        self.edges = []
        self.things = []
        self.constraints = []

class Edge:
    def __init__(self, name, from_node, commands, to_node, description=None, health=None, constraints=None):
        self.from_node = from_node 
        self.commands = commands
        self.to_node = to_node
        self.description = description
        self.health = health
        self.name = name
        self.constraints = constraints

class Constraint:
    def __init__(self, name, thing_name, state_name, ranking=None):
        self.name = name
        self.thing_name = thing_name
        self.state_name = state_name
        self.ranking = ranking

class Thing:
    def __init__(self, loc_node_name, unique_name, short_desc, description, weight=None):
        self.loc_node_name = loc_node_name
        self.unique_name = unique_name
        self.short_desc = short_desc
        self.description = description
        self.weight = weight

class Engine:
    def __init__(self, filename):
        self.filename = filename
        self.initial_state = None
        self.counter = 0
        self.initial_time = None
        self.health = 10

    def load(self):
        self.states = {}
        self.constraints = []
        self.things = []
        edges = []
        constraints = []
        fp = open(self.filename)
        for line in fp:
            if len(line) == 0 or line[0] == '#':
                continue
            items = line.strip().split("::")
            if items[0] == 'node':
                s = State(*items[1:]) #asterisk passes in stuff as instances
                if self.initial_state is None:
                    self.initial_state = s
                self.states[items[1]] = s
            elif items[0] == 'edge':
                edges.append(Edge(*items[1:]))
            elif items[0] == 'thing':
                self.things.append(Thing(*items[1:]))
            elif items[0] == 'constraint':
                self.constraints.append(Constraint(*items[1:]))
        for edge in edges:
            self.states[edge.from_node].edges.append(edge)            

    def play(self):
        current_state = self.initial_state
        while True:
            c_s = linebreak(current_state.description)
            print(c_s)
            command = input("Command> ")
            if command is None or command is " ":
                break
            splitcommand = command.split()
            if self.health == 0:
                return print("Game over!")
            #print(self.health)
            if command:
                for edge in current_state.edges: #bonus - add in health!
                    if command in edge.commands:
                        if edge.constraints:
                            for constraint in self.constraints:
                                if constraint.name in edge.constraints:
                                    for thing in self.things:
                                        if constraint.thing_name in thing.unique_name:
                                            if constraint.state_name in thing.loc_node_name:
                                                current_state = self.states[edge.to_node]
                                                if edge.description:
                                                    print(edge.description)
                                                    break
                        else:
                            current_state = self.states[edge.to_node]
                            if edge.description:
                                print(edge.description)
                                break
                    if splitcommand[0] in ['help']:
                        print(edge.name,":",edge.commands)
                   # if edge.health:
                   #     self.health = self.health + edge.health
                if splitcommand[0] in ['edge','edges']:
                    print(edge.description)
            if splitcommand:
                if splitcommand[0] in ['look','examine','inspect','stare']:
                    for i in splitcommand:
                        for thing in self.things:
                            if current_state.name in thing.loc_node_name:
                                if i in thing.unique_name:                                
                                    print(thing.description)
                if splitcommand[0] in ['get','pick','talk']:
                    for i in splitcommand:
                        for thing in self.things:
                            if i in thing.unique_name:
                                thing.loc_node_name = "backpack"
                                print("You obtained the",thing.unique_name,".")
                if splitcommand[0] in ['drop','put']:
                    for i in splitcommand:
                        for thing in self.things:
                            if i in thing.unique_name:
                                thing.loc_node_name = current_state.name
                                print("You dropped the",thing.unique_name,".")
            for word in splitcommand:
                if 'backpack' in splitcommand:
                    print("You have in your backpack:")
                    for thing in self.things:
                        if 'backpack' in thing.loc_node_name:
                            print(thing.unique_name,":",thing.description)
            for thing in self.things:
                if current_state.name in thing.loc_node_name:                    
                    print("You see",thing.short_desc,"(",thing.unique_name,").")

game = Engine("DoctorWho.txt")
game.load()
game.play()
